/**
  * @file <loop-functions/IcraLoopFunc.cpp>
  *
  * @author Antoine Ligot - <aligot@ulb.ac.be>
  *
  * @license MIT License
  */

#include "PstCtLoopFunc.h"

/****************************************/
/****************************************/

PstCtLoopFunction::PstCtLoopFunction() {
    m_unClock = 0;
    m_fObjectiveFunction = 0;
}

/****************************************/
/****************************************/

PstCtLoopFunction::PstCtLoopFunction(const PstCtLoopFunction& orig) {
}

/****************************************/
/****************************************/

PstCtLoopFunction::~PstCtLoopFunction() {}

/****************************************/
/****************************************/

void PstCtLoopFunction::Destroy() {

    m_tRobotStates.clear();
    RemoveArena();
}

/****************************************/
/****************************************/

void PstCtLoopFunction::Init(TConfigurationNode& t_tree) {

    CoreLoopFunctions::Init(t_tree);
    TConfigurationNode cParametersNode;
    try {
      cParametersNode = GetNode(t_tree, "params");
      GetNodeAttributeOrDefault(cParametersNode, "build_arena", m_bBuildArena, (bool) false);
      GetNodeAttributeOrDefault(cParametersNode, "number_edges", m_unNumberEdges, (UInt32) 3);
      GetNodeAttributeOrDefault(cParametersNode, "number_boxes_per_edge", m_unNumberBoxes, (UInt32) 1);
      GetNodeAttributeOrDefault(cParametersNode, "lenght_boxes", m_fLenghtBoxes, (Real) 0.25);
      GetNodeAttributeOrDefault(cParametersNode, "maximization", m_bMaximization, (bool) false);
    } catch(std::exception e) {
    }

    if (m_bBuildArena == true)
    {
        m_fDistributionRadius = GetArenaRadious();
        PositionArena();
    }

    InitRobotStates();
    m_pcArena->SetArenaColor(CColor::BLACK);

}

/****************************************/
/****************************************/

void PstCtLoopFunction::Reset() {
    CoreLoopFunctions::Reset();

    m_unClock = 0;
    m_fObjectiveFunction = 0;

    m_tRobotStates.clear();

    InitRobotStates();
    m_pcArena->SetArenaColor(CColor::BLACK);
}

/****************************************/
/****************************************/

void PstCtLoopFunction::PostStep() {

    m_unClock = GetSpace().GetSimulationClock();

    // ScoreControl();

}

/****************************************/
/****************************************/

void PstCtLoopFunction::PostExperiment() {
    
    ScoreControl();

    if (m_bMaximization == true){
        LOG << -m_fObjectiveFunction << std::endl;
    }
    else {
        LOG << m_fObjectiveFunction << std::endl;
    }
}

/****************************************/
/****************************************/

Real PstCtLoopFunction::GetObjectiveFunction() {
    if (m_bMaximization == true){
        return -m_fObjectiveFunction;
    }
    else {
        return m_fObjectiveFunction;
    }
}

/****************************************/
/****************************************/

void PstCtLoopFunction::ArenaControl() {

    return;
}

/****************************************/
/****************************************/

void PstCtLoopFunction::ScoreControl(){

    m_fObjectiveFunction += GetSheperdingScore();
}

/****************************************/
/****************************************/

Real PstCtLoopFunction::GetSheperdingScore() {

    UpdateRobotPositions();

    /*
    Real fScore = 0;
    Real fDistance = 0;
    TRobotStateMap::iterator it;
    TRobotStateMap::iterator jt;
    for (it = m_tRobotStates.begin(); it != m_tRobotStates.end(); ++it) {
        for (jt = it; jt != m_tRobotStates.end(); ++jt) {
            if (it != jt){
                if (it->second.sRobotType == "sheep" && jt->second.sRobotType == "sheep") {
                    Real d = (it->second.cPosition - jt->second.cPosition).Length();
                    fDistance = fDistance + d;
                }
            }
        }
    }
    fScore = fDistance / 10;
    */

    Real fScore = 0;
    Real fDistance = 0;
    Real fCentroidX = 0;
    Real fCentroidY = 0;
    Real unSheepCounter = 0;
    CVector2 cCentroid = CVector2(0, 0);

    TRobotStateMap::iterator it;

    for (it = m_tRobotStates.begin(); it != m_tRobotStates.end(); ++it) {

        if (it->second.sRobotType == "sheep") {
            fCentroidX += (it->second.cPosition.GetX());
            fCentroidY += (it->second.cPosition.GetY());
            unSheepCounter += 1;
        }
    }

    if (unSheepCounter > 0){
        fCentroidX = fCentroidX / unSheepCounter;
        fCentroidY = fCentroidY / unSheepCounter;
        cCentroid = CVector2(fCentroidX, fCentroidY);
    }

    for (it = m_tRobotStates.begin(); it != m_tRobotStates.end(); ++it) {
        if (it->second.sRobotType == "sheep") {
            Real d = (it->second.cPosition - cCentroid).Length();
            fDistance += d;
        }
    }

    fScore = fDistance / unSheepCounter;

    return fScore;
}

/****************************************/
/****************************************/

argos::CColor PstCtLoopFunction::GetFloorColor(const argos::CVector2& c_position_on_plane) {

    return CColor::GRAY50;
}


/****************************************/
/****************************************/

void PstCtLoopFunction::UpdateRobotPositions() {
    CSpace::TMapPerType& tEpuckMap = GetSpace().GetEntitiesByType("epuck");
    CVector2 cEpuckPosition(0,0);
    for (CSpace::TMapPerType::iterator it = tEpuckMap.begin(); it != tEpuckMap.end(); ++it) {
        CEPuckEntity* pcEpuck = any_cast<CEPuckEntity*>(it->second);
        cEpuckPosition.Set(pcEpuck->GetEmbodiedEntity().GetOriginAnchor().Position.GetX(),
                           pcEpuck->GetEmbodiedEntity().GetOriginAnchor().Position.GetY());

        m_tRobotStates[pcEpuck].cLastPosition = m_tRobotStates[pcEpuck].cPosition;
        m_tRobotStates[pcEpuck].cPosition = cEpuckPosition;
    }
}

/****************************************/
/****************************************/

void PstCtLoopFunction::InitRobotStates() {

    CSpace::TMapPerType& tEpuckMap = GetSpace().GetEntitiesByType("epuck");
    CVector2 cEpuckPosition(0,0);
    for (CSpace::TMapPerType::iterator it = tEpuckMap.begin(); it != tEpuckMap.end(); ++it) {
        CEPuckEntity* pcEpuck = any_cast<CEPuckEntity*>(it->second);
        cEpuckPosition.Set(pcEpuck->GetEmbodiedEntity().GetOriginAnchor().Position.GetX(),
                           pcEpuck->GetEmbodiedEntity().GetOriginAnchor().Position.GetY());

        m_tRobotStates[pcEpuck].cLastPosition = cEpuckPosition;
        m_tRobotStates[pcEpuck].cPosition = cEpuckPosition;
        m_tRobotStates[pcEpuck].unItem = 0;
        m_tRobotStates[pcEpuck].sRobotType = pcEpuck->GetId().substr(0,5);
    }
}

/****************************************/
/****************************************/

CVector3 PstCtLoopFunction::GetRandomPosition() {
  Real temp;
  Real a = m_pcRng->Uniform(CRange<Real>(0.0f, 1.0f));
  Real b = m_pcRng->Uniform(CRange<Real>(0.0f, 1.0f));
  Real c = m_pcRng->Uniform(CRange<Real>(-1.0f, 1.0f));
  Real d = m_pcRng->Uniform(CRange<Real>(-1.0f, 1.0f));
  // If b < a, swap them
  if (b < a) {
    temp = a;
    a = b;
    b = temp;
  }
  m_fDistributionRadius = 0.35;
  Real fPosX = (c * m_fDistributionRadius) + m_fDistributionRadius * cos(2 * CRadians::PI.GetValue() * (a/b));
  Real fPosY = (d * m_fDistributionRadius) + m_fDistributionRadius * sin(2 * CRadians::PI.GetValue() * (a/b));

  return CVector3(fPosX, fPosY, 0);
}

/****************************************/
/****************************************/

void PstCtLoopFunction::PositionArena() {
  CArenaEntity* pcArena;
  /*
    pcArena = new CArenaEntity("arena",
                               CVector3(0,0,0),
                               CQuaternion().FromEulerAngles(CRadians::ZERO,CRadians::ZERO,CRadians::ZERO), // TODO
                               CVector3(0.01,m_fLenghtBoxes,0.1),
                               "leds",
                               m_unNumberBoxes,
                               m_unNumberEdges,
                               0.017f,
                               1.0f);   */ // arena with 12 leds per block

  pcArena = new CArenaEntity("arena",
                             CVector3(0,0,0),
                             CQuaternion().FromEulerAngles(CRadians::ZERO,CRadians::ZERO,CRadians::ZERO), // TODO
                             CVector3(0.01,m_fLenghtBoxes,0.1),
                             "leds",
                             m_unNumberBoxes,
                             m_unNumberEdges,
                             0.125f,
                             1.0f);

  AddEntity(*pcArena);
  m_pcArena = pcArena;
}

/****************************************/
/****************************************/

void PstCtLoopFunction::RemoveArena() {
    std::ostringstream id;
    id << "arena";
    RemoveEntity(id.str().c_str());
}

/****************************************/
/****************************************/

Real PstCtLoopFunction::GetArenaRadious() {

    Real fRadious;
    fRadious =  (m_fLenghtBoxes*m_unNumberBoxes) / (2 * Tan(CRadians::PI / m_unNumberEdges));
    //fRadious = fRadious - 0.10; // Avoids to place robots close the walls.
    fRadious = fRadious - 0.65; // Reduced cluster at the begining

    return fRadious;
}

/****************************************/
/****************************************/

bool PstCtLoopFunction::IsEven(UInt32 unNumber) {
    bool even;
    if((unNumber%2)==0)
       even = true;
    else
       even = false;

    return even;
}

/****************************************/
/****************************************/

REGISTER_LOOP_FUNCTIONS(PstCtLoopFunction, "pst_ct_loop_function");
